@@ -44,7 +44,11 @@
 #' if calling [build()]. __Note__: Even if `build = TRUE`, if no edges or
 #' nodes are provided, the graph will not be built and the pointer will be
 #' `NULL`.
-#' @param class Character; one of `"UNKNOWN"`, `"DAG"`, `"PDAG"`, or `"UG"`.
+#' @param class Character; one of `"AUTO"`, `"DAG"`, `"UG"`, `"PDAG"`, `"ADMG"`,
+#' `"AG"`, or `"UNKNOWN"`. `"AUTO"` will automatically pick the appropriate
+#' class based on the first match in the order of `"DAG"`, `"UG"`, `"PDAG"`,
+#' `"ADMG"`, and `"AG"`.
+#' It will default to `"UNKNOWN"` if no match is found.
 #' @param state For internal use. Build a graph by supplying a pre-constructed
 #' state environment.
 #'
@@ -96,7 +100,7 @@
 #'   build = FALSE
 #' )
 #'
-#' cg@built # FALSE
+#' cg5@built # FALSE
 #'
 #' @family caugi
 #' @concept caugi
@@ -111,7 +115,8 @@ caugi <- S7::new_class(
       S7::class_any,
       getter = function(self) self@`.state`$nodes,
       setter = function(self, value) {
-        stop("nodes is read-only via @ <-. ",
+        stop(
+          "nodes is read-only via @ <-. ",
           "Use `add_nodes()` or `remove_nodes()` instead. ",
           "Advanced users can modify `cg@.state$nodes` directly, ",
           "but this is not recommended.",
@@ -123,7 +128,8 @@ caugi <- S7::new_class(
       S7::class_any,
       getter = function(self) self@`.state`$edges,
       setter = function(self, value) {
-        stop("`edges` properrt is read-only via @ <-. ",
+        stop(
+          "`edges` property is read-only via @ <-. ",
           "Use `add_edges()` or `remove_edges()` instead. ",
           "Advanced users can modify `cg@.state$edges` directly ",
           "but this is not recommended.",
@@ -135,7 +141,8 @@ caugi <- S7::new_class(
       S7::class_any,
       getter = function(self) self@`.state`$ptr,
       setter = function(self, value) {
-        stop("`ptr` property is read-only via @ <-. ",
+        stop(
+          "`ptr` property is read-only via @ <-. ",
           "Use `build()` to (re)build the graph.",
           call. = FALSE
         )
@@ -150,12 +157,17 @@ caugi <- S7::new_class(
         }
         # extra precuation
         if (self@`.state`$simple != is_simple_ptr(self@ptr)) {
-          stop(paste0(
-            "Internal warning: graph simplicity mismatch between R (",
-            self@`.state`$simple, ") and Rust (", is_simple_ptr(self@ptr),
-            "). ",
-            "Please report this issue."
-          ), call. = FALSE)
+          stop(
+            paste0(
+              "Internal warning: graph simplicity mismatch between R (",
+              self@`.state`$simple,
+              ") and Rust (",
+              is_simple_ptr(self@ptr),
+              "). ",
+              "Please report this issue."
+            ),
+            call. = FALSE
+          )
         }
         is_simple_ptr(self@ptr)
       }
@@ -164,7 +176,8 @@ caugi <- S7::new_class(
       S7::class_logical,
       getter = function(self) self@`.state`$built,
       setter = function(self, value) {
-        stop("`built` property is read-only via @ <-. ",
+        stop(
+          "`built` property is read-only via @ <-. ",
           "It should only be set at construction or when calling `build()`.",
           call. = FALSE
         )
@@ -178,18 +191,26 @@ caugi <- S7::new_class(
           return(self@`.state`$class)
         }
         # extra precuation
-        if (toupper(self@`.state`$class) != toupper(graph_class_ptr(self@ptr))) {
-          stop(paste0(
-            "Internal warning: graph class mismatch between R (",
-            self@`.state`$class, ") and Rust (", graph_class_ptr(self@ptr),
-            "). ",
-            "Please report this issue."
-          ), call. = FALSE)
+        if (
+          toupper(self@`.state`$class) != toupper(graph_class_ptr(self@ptr))
+        ) {
+          stop(
+            paste0(
+              "Internal warning: graph class mismatch between R (",
+              self@`.state`$class,
+              ") and Rust (",
+              graph_class_ptr(self@ptr),
+              "). ",
+              "Please report this issue."
+            ),
+            call. = FALSE
+          )
         }
         graph_class_ptr(self@ptr)
       },
       setter = function(self, value) {
-        stop("`graph_class` property is read-only via @ <-. ",
+        stop(
+          "`graph_class` property is read-only via @ <-. ",
           "It should only be set at construction.",
           call. = FALSE
         )
@@ -201,7 +222,8 @@ caugi <- S7::new_class(
         return(self@`.state`$name_index_map)
       },
       setter = function(self, value) {
-        stop("`name_index_map` property is read-only via @ <-. ",
+        stop(
+          "`name_index_map` property is read-only via @ <-. ",
           "It is managed internally.",
           call. = FALSE
         )
@@ -210,8 +232,14 @@ caugi <- S7::new_class(
   ),
   validator = function(self) {
     s <- self@`.state`
-    if (isFALSE(s$simple) && !identical(s$class, "UNKNOWN")) {
-      return("If simple = FALSE, class must be 'UNKNOWN'")
+    # Allow simple = FALSE for UNKNOWN, ADMG, and AG (mixed edges can share pairs)
+    if (
+      isFALSE(s$simple) &&
+        !identical(s$class, "UNKNOWN") &&
+        !identical(s$class, "ADMG") &&
+        !identical(s$class, "AG")
+    ) {
+      return("If simple = FALSE, class must be 'UNKNOWN', 'ADMG', or 'AG'")
     }
 
     if (is.null(self@ptr) && s$built) {
@@ -225,15 +253,21 @@ caugi <- S7::new_class(
     if (!is.null(self@ptr) && s$built) {
       if (toupper(graph_class_ptr(self@ptr)) != toupper(s$class)) {
         return(paste0(
-          "Internal error: graph class mismatch between R (", s$class,
-          ") and Rust (", graph_class_ptr(self@ptr), "). ",
+          "Internal error: graph class mismatch between R (",
+          s$class,
+          ") and Rust (",
+          graph_class_ptr(self@ptr),
+          "). ",
           "Please report this issue."
         ))
       }
       if (is_simple_ptr(self@ptr) != s$simple) {
         return(paste0(
-          "Internal error: graph simplicity mismatch between R (", s$simple,
-          ") and Rust (", is_simple_ptr(self@ptr), "). ",
+          "Internal error: graph simplicity mismatch between R (",
+          s$simple,
+          ") and Rust (",
+          is_simple_ptr(self@ptr),
+          "). ",
           "Please report this issue."
         ))
       }
@@ -242,14 +276,18 @@ caugi <- S7::new_class(
     # todo: how may a user corrupt via setting cg@.state$field <- value?
     NULL
   },
-  constructor = function(...,
-                         from = NULL, edge = NULL, to = NULL,
-                         nodes = NULL,
-                         edges_df = NULL,
-                         simple = TRUE,
-                         build = TRUE,
-                         class = c("UNKNOWN", "DAG", "PDAG", "UG"),
-                         state = NULL) {
+  constructor = function(
+    ...,
+    from = NULL,
+    edge = NULL,
+    to = NULL,
+    nodes = NULL,
+    edges_df = NULL,
+    simple = TRUE,
+    build = TRUE,
+    class = c("AUTO", "DAG", "UG", "PDAG", "ADMG", "AG", "UNKNOWN"),
+    state = NULL
+  ) {
     if (!is.null(state)) {
       return(S7::new_object(
         caugi,
@@ -265,15 +303,14 @@ caugi <- S7::new_class(
     has_df <- !is.null(edges_df)
     if (has_df) {
       if (!is.data.frame(edges_df)) {
-        stop("`edges_df` must be a data.frame or data.table.",
-          call. = FALSE
-        )
+        stop("`edges_df` must be a data.frame or data.table.", call. = FALSE)
       }
       required_cols <- c("from", "edge", "to")
       if (!all(required_cols %chin% colnames(edges_df))) {
         stop(
           "`edges_df` must contain columns: ",
-          paste(required_cols, collapse = ", "), ".",
+          paste(required_cols, collapse = ", "),
+          ".",
           call. = FALSE
         )
       }
@@ -333,21 +370,25 @@ caugi <- S7::new_class(
       )
     }
 
-
     if (!is.null(nodes)) {
       if (!is.character(nodes)) {
         if (is.data.frame(nodes) && "name" %in% colnames(nodes)) {
           nodes <- as.character(nodes$name)
         } else {
-          stop("`nodes` must be a character vector of node names.",
+          stop(
+            "`nodes` must be a character vector of node names.",
             call. = FALSE
           )
         }
       }
     }
 
-    if (!simple && class != "UNKNOWN") {
-      stop("If simple = FALSE, class must be 'UNKNOWN'", call. = FALSE)
+    # Allow simple = FALSE for UNKNOWN, ADMG, and AG (mixed edges can share pairs)
+    if (!simple && class != "UNKNOWN" && class != "ADMG" && class != "AG") {
+      stop(
+        "If simple = FALSE, class must be 'UNKNOWN', 'ADMG', or 'AG'",
+        call. = FALSE
+      )
     }
 
     # Parse into edges + declared nodes
@@ -371,7 +412,8 @@ caugi <- S7::new_class(
       declared <- nodes
     } else {
       if (build == TRUE && !missing(build)) {
-        warning("No edges or nodes provided; graph will not be built.",
+        warning(
+          "No edges or nodes provided; graph will not be built.",
           call. = FALSE
         )
       }
@@ -418,6 +460,11 @@ caugi <- S7::new_class(
 
       gptr <- graph_builder_build_view(b, class)
       built <- TRUE
+
+      # resolve AUTO to actual class from Rust
+      if (class == "AUTO") {
+        class <- graph_class_ptr(gptr)
+      }
     }
 
     # initialize fastmap for name to index mapping
@@ -461,14 +508,16 @@ caugi <- S7::new_class(
 #'
 #' @keywords internal
 .view_to_caugi <- function(ptr, node_names = NULL) {
-  if (is.null(ptr)) stop("ptr is NULL", call. = FALSE)
+  if (is.null(ptr)) {
+    stop("ptr is NULL", call. = FALSE)
+  }
 
   n <- n_ptr(ptr)
-  if (is.null(node_names)) node_names <- sprintf("V%d", seq_len(n))
+  if (is.null(node_names)) {
+    node_names <- sprintf("V%d", seq_len(n))
+  }
   if (length(node_names) != n) {
-    stop("length(node_names) must equal n_ptr(ptr)",
-      call. = FALSE
-    )
+    stop("length(node_names) must equal n_ptr(ptr)", call. = FALSE)
   }
 
   edges_idx <- edges_ptr_df(ptr)
@@ -487,10 +536,13 @@ caugi <- S7::new_class(
   nodes_tbl <- .node_constructor(names = node_names)
 
   name_index_map <- fastmap::fastmap()
-  do.call(name_index_map$mset, .set_names(
-    as.list(seq_len(n) - 1L),
-    node_names
-  ))
+  do.call(
+    name_index_map$mset,
+    .set_names(
+      as.list(seq_len(n) - 1L),
+      node_names
+    )
+  )
 
   state <- .cg_state(
     nodes = nodes_tbl,
@@ -516,15 +568,22 @@ caugi <- S7::new_class(
 #' @param built Logical; whether the graph has been built.
 #' @param simple Logical; whether the graph is simple
 #' (no parallel edges or self-loops).
-#' @param class Character; one of `"UNKNOWN"`, `"DAG"`, or `"PDAG"`.
+#' @param class Character; one of `"UNKNOWN"`, `"DAG"`, `"UG"`, `"PDAG"`, `"ADMG"`, or `"AG"`.
 #' @param name_index_map A `fastmap` mapping node names to their zero indexed
 #' indices.
 #'
 #' @returns An environment containing the graph state.
 #'
 #' @keywords internal
-.cg_state <- function(nodes, edges, ptr, built, simple, class,
-                      name_index_map, index_name_map) {
+.cg_state <- function(
+  nodes,
+  edges,
+  ptr,
+  built,
+  simple,
+  class,
+  name_index_map
+) {
   e <- new.env(parent = emptyenv())
   e$nodes <- nodes
   e$edges <- edges
