@@ -1,120 +1,175 @@
 // SPDX-License-Identifier: MIT
 //! Class-agnostic Structural Hamming Distance over CaugiGraph.
 
-use crate::{edges::EdgeClass, graph::CaugiGraph};
+use rustc_hash::FxHashMap;
 
-#[derive(PartialEq, Eq)]
-enum PairKind {
-    None,
+use crate::{
+    edges::{EdgeClass, Mark},
+    graph::CaugiGraph,
+};
+
+/// Represents the semantic kind of an edge between two nodes.
+/// - `Sym`: A symmetric edge (e.g., undirected `---` or bidirected `<->`).
+/// - `Asym`: An asymmetric edge with direction; `is_outgoing=true` means the edge
+///   points toward the neighbor (i.e., neighbor has Arrow mark).
+#[derive(PartialEq, Eq, Clone, Copy)]
+enum EdgeKind {
     Sym(EdgeClass),
     Asym(EdgeClass, bool),
 }
 
+/// Compute the EdgeKind for an edge at position `idx` in the graph's CSR storage.
 #[inline]
-fn pair_kind(core: &CaugiGraph, v: u32, mut k: usize, end: usize) -> (PairKind, usize) {
-    while k < end {
-        let c = core.col_index[k];
-        if c < v {
-            k += 1;
-            continue;
-        }
-        if c > v {
-            return (PairKind::None, k);
-        }
-        let spec = &core.registry.specs[core.etype[k] as usize];
-        let kind = if spec.symmetric {
-            PairKind::Sym(spec.class)
+fn compute_edge_kind(graph: &CaugiGraph, idx: usize) -> EdgeKind {
+    let spec = &graph.registry.specs[graph.etype[idx] as usize];
+    if spec.symmetric {
+        EdgeKind::Sym(spec.class)
+    } else {
+        // Determine semantic direction: "outgoing" means neighbor has Arrow mark.
+        // side[idx] stores position: 0 = tail position, 1 = head position.
+        // If side=0, neighbor mark = spec.head; if side=1, neighbor mark = spec.tail.
+        let neighbor_mark = if graph.side[idx] == 0 {
+            spec.head
         } else {
-            PairKind::Asym(spec.class, core.side[k] == 0)
+            spec.tail
         };
-        return (kind, k + 1);
+        let is_outgoing = neighbor_mark == Mark::Arrow;
+        EdgeKind::Asym(spec.class, is_outgoing)
     }
-    (PairKind::None, k)
 }
 
+/// Advance cursor through a sorted adjacency list to find neighbor `target`.
+/// Returns `(Some(edge_kind), new_cursor)` if found, `(None, new_cursor)` otherwise.
+/// The cursor is advanced past any neighbors less than `target`.
 #[inline]
-fn pair_any(core: &CaugiGraph, v: u32, mut k: usize, end: usize) -> (bool, usize) {
-    while k < end {
-        let c = core.col_index[k];
-        if c < v {
-            k += 1;
-            continue;
+fn find_edge_kind(
+    graph: &CaugiGraph,
+    target: u32,
+    mut cursor: usize,
+    row_end: usize,
+) -> (Option<EdgeKind>, usize) {
+    while cursor < row_end {
+        let neighbor = graph.col_index[cursor];
+        if neighbor < target {
+            cursor += 1;
+        } else if neighbor == target {
+            return (Some(compute_edge_kind(graph, cursor)), cursor + 1);
+        } else {
+            // neighbor > target: target not in this row
+            return (None, cursor);
         }
-        if c > v {
-            return (false, k);
-        }
-        return (true, k + 1); // found any half-edge between u and v
     }
-    (false, k)
+    (None, cursor)
 }
 
-pub fn shd(t: &CaugiGraph, g: &CaugiGraph) -> (f64, usize) {
-    shd_with_perm(t, g, &(0..t.n()).collect::<Vec<_>>())
+/// Advance cursor to check if any edge exists to `target` in a sorted adjacency list.
+/// Returns `(true, new_cursor)` if found, `(false, new_cursor)` otherwise.
+#[inline]
+fn has_edge_to(
+    graph: &CaugiGraph,
+    target: u32,
+    mut cursor: usize,
+    row_end: usize,
+) -> (bool, usize) {
+    while cursor < row_end {
+        let neighbor = graph.col_index[cursor];
+        if neighbor < target {
+            cursor += 1;
+        } else if neighbor == target {
+            return (true, cursor + 1);
+        } else {
+            return (false, cursor);
+        }
+    }
+    (false, cursor)
 }
 
-pub fn shd_with_perm(t: &CaugiGraph, g: &CaugiGraph, perm: &[u32]) -> (f64, usize) {
-    assert_eq!(t.n(), g.n(), "graph size mismatch");
-    assert_eq!(perm.len() as u32, t.n(), "perm length mismatch");
-    let n = t.n() as usize;
+/// Compute Structural Hamming Distance between two graphs with identical node ordering.
+pub fn shd(truth: &CaugiGraph, guess: &CaugiGraph) -> (f64, usize) {
+    shd_with_perm(truth, guess, &(0..truth.n()).collect::<Vec<_>>())
+}
+
+/// Compute Structural Hamming Distance with a node permutation mapping.
+/// `perm[i]` gives the index in `guess` that corresponds to node `i` in `truth`.
+pub fn shd_with_perm(truth: &CaugiGraph, guess: &CaugiGraph, perm: &[u32]) -> (f64, usize) {
+    assert_eq!(truth.n(), guess.n(), "graph size mismatch");
+    assert_eq!(perm.len() as u32, truth.n(), "perm length mismatch");
+    let n = truth.n() as usize;
     if n <= 1 {
         return (0.0, 0);
     }
-    let mut dist = 0usize;
+
+    let mut distance = 0usize;
+    // Reuse HashMap across iterations to avoid repeated allocations.
+    // FxHashMap is faster than std HashMap for integer keys.
+    let mut guess_edges: FxHashMap<u32, EdgeKind> = FxHashMap::default();
+
     for u in 0..n {
-        let (ts, te) = {
-            let r = t.row_range(u as u32);
-            (r.start, r.end)
-        };
-        let u_g = perm[u] as usize;
-        let (gs, ge) = {
-            let r = g.row_range(u_g as u32);
-            (r.start, r.end)
-        };
-        let (mut kt, mut kg) = (ts, gs);
+        let row_t = truth.row_range(u as u32);
+        let row_start = row_t.start;
+        let row_end = row_t.end;
+        let u_in_guess = perm[u];
+
+        // Build a map of guess's edges for this row.
+        // Necessary because perm[v] values are not in sorted order,
+        // so we cannot use the sorted cursor traversal optimization.
+        guess_edges.clear();
+        for idx in guess.row_range(u_in_guess) {
+            let neighbor = guess.col_index[idx];
+            let kind = compute_edge_kind(guess, idx);
+            guess_edges.insert(neighbor, kind);
+        }
+
+        let mut cursor_t = row_start;
         for v in (u + 1)..n {
-            let (a, kt2) = pair_kind(t, v as u32, kt, te);
-            let v_g = perm[v];
-            let (b, kg2) = pair_kind(g, v_g, kg, ge);
-            if a != b {
-                dist += 1;
+            let (edge_in_truth, new_cursor) = find_edge_kind(truth, v as u32, cursor_t, row_end);
+            cursor_t = new_cursor;
+
+            let v_in_guess = perm[v];
+            let edge_in_guess = guess_edges.get(&v_in_guess).copied();
+
+            if edge_in_truth != edge_in_guess {
+                distance += 1;
             }
-            kt = kt2;
-            kg = kg2;
         }
     }
-    let comps = n * (n - 1) / 2;
-    ((dist as f64) / (comps as f64), dist)
+
+    let num_pairs = n * (n - 1) / 2;
+    ((distance as f64) / (num_pairs as f64), distance)
 }
 
-pub fn hd(t: &CaugiGraph, g: &CaugiGraph) -> (f64, usize) {
-    assert_eq!(t.n(), g.n(), "graph size mismatch");
-    let n = t.n() as usize;
+/// Compute skeleton Hamming Distance (ignores edge types and directions).
+pub fn hd(truth: &CaugiGraph, guess: &CaugiGraph) -> (f64, usize) {
+    assert_eq!(truth.n(), guess.n(), "graph size mismatch");
+    let n = truth.n() as usize;
     if n <= 1 {
         return (0.0, 0);
     }
-    let mut dist = 0usize;
+
+    let mut distance = 0usize;
     for u in 0..n {
-        let (ts, te) = {
-            let r = t.row_range(u as u32);
-            (r.start, r.end)
-        };
-        let (gs, ge) = {
-            let r = g.row_range(u as u32);
-            (r.start, r.end)
-        };
-        let (mut kt, mut kg) = (ts, gs);
+        let row_t = truth.row_range(u as u32);
+        let row_g = guess.row_range(u as u32);
+
+        let mut cursor_t = row_t.start;
+        let mut cursor_g = row_g.start;
+
         for v in (u + 1)..n {
-            let (a, kt2) = pair_any(t, v as u32, kt, te);
-            let (b, kg2) = pair_any(g, v as u32, kg, ge);
-            if a ^ b {
-                dist += 1;
+            let (has_edge_t, new_cursor_t) = has_edge_to(truth, v as u32, cursor_t, row_t.end);
+            let (has_edge_g, new_cursor_g) = has_edge_to(guess, v as u32, cursor_g, row_g.end);
+
+            // XOR: count as difference if exactly one graph has the edge
+            if has_edge_t ^ has_edge_g {
+                distance += 1;
             }
-            kt = kt2;
-            kg = kg2;
+
+            cursor_t = new_cursor_t;
+            cursor_g = new_cursor_g;
         }
     }
-    let comps = n * (n - 1) / 2;
-    ((dist as f64) / (comps as f64), dist)
+
+    let num_pairs = n * (n - 1) / 2;
+    ((distance as f64) / (num_pairs as f64), distance)
 }
 
 #[cfg(feature = "gadjid")]
@@ -590,13 +645,11 @@ mod tests {
         let mut reg = EdgeRegistry::new();
         reg.register_builtins().unwrap();
         let d = reg.code_of("-->").unwrap();
-        let u = reg.code_of("---").unwrap();
 
-        // CPDAG: 0 -> 1, 1 -- 2
+        // CPDAG: v-structure 0 -> 2 <- 1 (collider at 2)
         let mut b = GraphBuilder::new_with_registry(3, true, &reg);
-        b.add_edge(0, 1, d).unwrap();
         b.add_edge(0, 2, d).unwrap();
-        b.add_edge(1, 2, u).unwrap();
+        b.add_edge(1, 2, d).unwrap();
         let p = Pdag::new(std::sync::Arc::new(b.finalize().unwrap())).unwrap();
         assert!(p.is_cpdag());
 
@@ -630,21 +683,18 @@ mod tests {
         let mut reg = EdgeRegistry::new();
         reg.register_builtins().unwrap();
         let d = reg.code_of("-->").unwrap();
-        let u = reg.code_of("---").unwrap();
 
-        // True CPDAG: 0 -> 1, 1 -- 2
+        // True CPDAG: v-structure 0 -> 2 <- 1 (collider at 2)
         let mut bt = GraphBuilder::new_with_registry(3, true, &reg);
-        bt.add_edge(0, 1, d).unwrap();
         bt.add_edge(0, 2, d).unwrap();
-        bt.add_edge(1, 2, u).unwrap();
+        bt.add_edge(1, 2, d).unwrap();
         let p_true = Pdag::new(std::sync::Arc::new(bt.finalize().unwrap())).unwrap();
         assert!(p_true.is_cpdag());
 
-        // Guess CPDAG: change 1 -- 2 into 1 -> 2
+        // Guess CPDAG: different v-structure 0 -> 1 <- 2 (collider at 1 instead of 2)
         let mut bg = GraphBuilder::new_with_registry(3, true, &reg);
         bg.add_edge(0, 1, d).unwrap();
-        bg.add_edge(0, 2, d).unwrap();
-        bg.add_edge(1, 2, d).unwrap();
+        bg.add_edge(2, 1, d).unwrap();
         let p_guess = Pdag::new(std::sync::Arc::new(bg.finalize().unwrap())).unwrap();
         assert!(p_guess.is_cpdag());
 
@@ -655,11 +705,11 @@ mod tests {
     }
 
     #[test]
-    fn shd_pair_kind_c_gt_v_branch() {
+    fn shd_finds_edge_when_neighbor_gt_target() {
         use crate::edges::EdgeRegistry;
         use crate::graph::builder::GraphBuilder;
 
-        // t: edge 0->2 only; g: empty. For pair (u=0,v=1), row has col=2>1 ⇒ early None.
+        // t: edge 0->2 only; g: empty. For pair (u=0,v=1), row has neighbor=2>1 ⇒ early None.
         let mut reg = EdgeRegistry::new();
         reg.register_builtins().unwrap();
         let d = reg.code_of("-->").unwrap();
@@ -675,11 +725,11 @@ mod tests {
     }
 
     #[test]
-    fn hd_pair_any_c_gt_v_branch() {
+    fn hd_returns_false_when_neighbor_gt_target() {
         use crate::edges::EdgeRegistry;
         use crate::graph::builder::GraphBuilder;
 
-        // Same shape as above. For pair_any at (0,1) we hit c>v ⇒ (false, k).
+        // Same shape as above. For has_edge_to at (0,1) we hit neighbor>target ⇒ false.
         let mut reg = EdgeRegistry::new();
         reg.register_builtins().unwrap();
         let d = reg.code_of("-->").unwrap();
@@ -738,9 +788,9 @@ mod tests {
         reg.register_builtins().unwrap();
         let d = reg.code_of("-->").unwrap();
 
-        // Build a valid CPDAG with n=3: 0->1, 1--2.
+        // Build a valid CPDAG with n=3: v-structure 0->2<-1
         let mut b = GraphBuilder::new_with_registry(3, true, &reg);
-        b.add_edge(0, 1, d).unwrap();
+        b.add_edge(0, 2, d).unwrap();
         b.add_edge(1, 2, d).unwrap();
         let p = Pdag::new(std::sync::Arc::new(b.finalize().unwrap())).unwrap();
         assert!(p.is_cpdag());
@@ -750,4 +800,163 @@ mod tests {
         let err = oset_aid_align(AidInput::Pdag(&p), AidInput::Pdag(&p), &bad_inv).unwrap_err();
         assert!(err.contains("index map length does not match graph size"));
     }
+
+    /// Test that shd_with_perm correctly handles non-identity permutations.
+    /// This is the exact bug case: same edges but different node orderings.
+    /// Graph t: nodes A=0, B=1, C=2, D=3 with edges A---B, A-->C, D-->C
+    /// Graph g: nodes D=0, C=1, A=2, B=3 with same logical edges
+    /// Permutation maps t's indices to g's: perm = [2, 3, 1, 0]
+    #[test]
+    fn shd_with_perm_different_node_order_same_edges() {
+        use crate::edges::EdgeRegistry;
+        use crate::graph::builder::GraphBuilder;
+
+        let mut reg = EdgeRegistry::new();
+        reg.register_builtins().unwrap();
+        let dir = reg.code_of("-->").unwrap();
+        let und = reg.code_of("---").unwrap();
+
+        // Graph t: A=0, B=1, C=2, D=3
+        // Edges: A---B (0---1), A-->C (0-->2), D-->C (3-->2)
+        let mut bt = GraphBuilder::new_with_registry(4, true, &reg);
+        bt.add_edge(0, 1, und).unwrap(); // A --- B
+        bt.add_edge(0, 2, dir).unwrap(); // A --> C
+        bt.add_edge(3, 2, dir).unwrap(); // D --> C
+        let t = bt.finalize().unwrap();
+
+        // Graph g: D=0, C=1, A=2, B=3
+        // Same logical edges: A---B (2---3), A-->C (2-->1), D-->C (0-->1)
+        let mut bg = GraphBuilder::new_with_registry(4, true, &reg);
+        bg.add_edge(2, 3, und).unwrap(); // A --- B
+        bg.add_edge(2, 1, dir).unwrap(); // A --> C
+        bg.add_edge(0, 1, dir).unwrap(); // D --> C
+        let g = bg.finalize().unwrap();
+
+        // perm[i] = index in g that corresponds to node i in t
+        // t: A=0 -> g: A=2, so perm[0] = 2
+        // t: B=1 -> g: B=3, so perm[1] = 3
+        // t: C=2 -> g: C=1, so perm[2] = 1
+        // t: D=3 -> g: D=0, so perm[3] = 0
+        let perm = [2u32, 3, 1, 0];
+
+        let (norm, count) = shd_with_perm(&t, &g, &perm);
+        assert_eq!(
+            count, 0,
+            "Same edges with different node order should have SHD=0"
+        );
+        assert_eq!(norm, 0.0);
+    }
+
+    /// Test shd_with_perm with reversed permutation.
+    #[test]
+    fn shd_with_perm_reversed_order() {
+        use crate::edges::EdgeRegistry;
+        use crate::graph::builder::GraphBuilder;
+
+        let mut reg = EdgeRegistry::new();
+        reg.register_builtins().unwrap();
+        let dir = reg.code_of("-->").unwrap();
+
+        // Graph t: 0->1->2
+        let mut bt = GraphBuilder::new_with_registry(3, true, &reg);
+        bt.add_edge(0, 1, dir).unwrap();
+        bt.add_edge(1, 2, dir).unwrap();
+        let t = bt.finalize().unwrap();
+
+        // Graph g: same structure but nodes reversed: 2->1->0
+        // In g's indexing: 0->1->2 (same as t)
+        // But we'll build it as if logical order is reversed
+        let mut bg = GraphBuilder::new_with_registry(3, true, &reg);
+        bg.add_edge(2, 1, dir).unwrap(); // logical 0->1
+        bg.add_edge(1, 0, dir).unwrap(); // logical 1->2
+        let g = bg.finalize().unwrap();
+
+        // perm: t's node i corresponds to g's node (2-i)
+        let perm = [2u32, 1, 0];
+
+        let (norm, count) = shd_with_perm(&t, &g, &perm);
+        assert_eq!(count, 0);
+        assert_eq!(norm, 0.0);
+    }
+
+    /// Test shd_with_perm correctly detects differences with permutation.
+    #[test]
+    fn shd_with_perm_detects_difference() {
+        use crate::edges::EdgeRegistry;
+        use crate::graph::builder::GraphBuilder;
+
+        let mut reg = EdgeRegistry::new();
+        reg.register_builtins().unwrap();
+        let dir = reg.code_of("-->").unwrap();
+        let und = reg.code_of("---").unwrap();
+
+        // Graph t: 0->1, 0->2
+        let mut bt = GraphBuilder::new_with_registry(3, true, &reg);
+        bt.add_edge(0, 1, dir).unwrap();
+        bt.add_edge(0, 2, dir).unwrap();
+        let t = bt.finalize().unwrap();
+
+        // Graph g: 0->1, 0---2 (undirected instead of directed)
+        // With permutation [1, 2, 0] (rotated)
+        // In g's ordering: node 1 in t -> node 2 in g, etc.
+        let mut bg = GraphBuilder::new_with_registry(3, true, &reg);
+        bg.add_edge(1, 2, dir).unwrap(); // corresponds to 0->1 in t
+        bg.add_edge(1, 0, und).unwrap(); // corresponds to 0---2 in t (DIFFERENT!)
+        let g = bg.finalize().unwrap();
+
+        let perm = [1u32, 2, 0];
+
+        let (_, count) = shd_with_perm(&t, &g, &perm);
+        assert_eq!(
+            count, 1,
+            "Should detect one difference: directed vs undirected"
+        );
+    }
+
+    /// Test shd_with_perm with larger graph and complex permutation.
+    #[test]
+    fn shd_with_perm_larger_graph() {
+        use crate::edges::EdgeRegistry;
+        use crate::graph::builder::GraphBuilder;
+
+        let mut reg = EdgeRegistry::new();
+        reg.register_builtins().unwrap();
+        let dir = reg.code_of("-->").unwrap();
+        let und = reg.code_of("---").unwrap();
+        let bi = reg.code_of("<->").unwrap();
+
+        // Graph t: 6 nodes with various edges
+        // 0->1, 1->2, 2---3, 3<->4, 4->5
+        let mut bt = GraphBuilder::new_with_registry(6, true, &reg);
+        bt.add_edge(0, 1, dir).unwrap();
+        bt.add_edge(1, 2, dir).unwrap();
+        bt.add_edge(2, 3, und).unwrap();
+        bt.add_edge(3, 4, bi).unwrap();
+        bt.add_edge(4, 5, dir).unwrap();
+        let t = bt.finalize().unwrap();
+
+        // Graph g: same edges but with permutation [3, 5, 1, 4, 0, 2]
+        // t's node 0 -> g's node 3
+        // t's node 1 -> g's node 5
+        // t's node 2 -> g's node 1
+        // t's node 3 -> g's node 4
+        // t's node 4 -> g's node 0
+        // t's node 5 -> g's node 2
+        let mut bg = GraphBuilder::new_with_registry(6, true, &reg);
+        bg.add_edge(3, 5, dir).unwrap(); // 0->1 in t
+        bg.add_edge(5, 1, dir).unwrap(); // 1->2 in t
+        bg.add_edge(1, 4, und).unwrap(); // 2---3 in t
+        bg.add_edge(4, 0, bi).unwrap(); // 3<->4 in t
+        bg.add_edge(0, 2, dir).unwrap(); // 4->5 in t
+        let g = bg.finalize().unwrap();
+
+        let perm = [3u32, 5, 1, 4, 0, 2];
+
+        let (norm, count) = shd_with_perm(&t, &g, &perm);
+        assert_eq!(
+            count, 0,
+            "Same edges with complex permutation should have SHD=0"
+        );
+        assert_eq!(norm, 0.0);
+    }
 }
