@@ -262,6 +262,23 @@ test_that("building PDAG with bidirected edges results in error", {
   )
 })
 
+# ──────────────────────────────────────────────────────────────────────────────
+# ──────────────────────────────── AUTO tests ──────────────────────────────────
+# ──────────────────────────────────────────────────────────────────────────────
+
+test_that("AUTO class picks the correct class", {
+  cg <- caugi(A %-->% B, class = "AUTO")
+  expect_equal(cg@graph_class, "DAG")
+  cg <- caugi(A %---% B, class = "AUTO")
+  expect_equal(cg@graph_class, "UG")
+  cg <- caugi(A %<->% B, class = "AUTO")
+  expect_equal(cg@graph_class, "ADMG")
+  cg <- caugi(A %o->% B, class = "AUTO")
+  expect_equal(cg@graph_class, "UNKNOWN")
+  cg <- caugi(A %-->% B %---% C, class = "AUTO")
+  expect_equal(cg@graph_class, "PDAG")
+})
+
 # ──────────────────────────────────────────────────────────────────────────────
 # ──────────────────────── Standard evaluation input ───────────────────────────
 # ──────────────────────────────────────────────────────────────────────────────
@@ -270,7 +287,7 @@ test_that("caugi builds from parallel vectors", {
   cg_vec <- caugi(
     from = c("A", "B", "C"),
     edge = c("-->", "---", "<->"),
-    to   = c("B", "C", "D")
+    to = c("B", "C", "D")
   )
 
   expect_s7_class(cg_vec, caugi)
@@ -290,7 +307,9 @@ test_that("caugi forbids mixing ... with from/edge/to", {
   expect_error(
     caugi(
       A %-->% B,
-      from = "C", edge = "-->", to = "D"
+      from = "C",
+      edge = "-->",
+      to = "D"
     ),
     "Provide edges via infix expressions"
   )
@@ -308,7 +327,7 @@ test_that("caugi requires from, edge, to all present and equal length", {
     caugi(
       from = c("A", "B"),
       edge = c("-->", "---"),
-      to   = "C"
+      to = "C"
     ),
     "`from`, `edge`, and `to` must be equal length."
   )
@@ -326,7 +345,7 @@ test_that("caugi(vector mode) gets the same result as in with DSL", {
   cg1 <- caugi(
     from = c("A"),
     edge = c("-->"),
-    to   = c("B")
+    to = c("B")
   )
   cg2 <- caugi(A %-->% B)
   expect_equal(cg1, cg2)
@@ -404,7 +423,7 @@ test_that("edges_df input works", {
   edges_df <- data.frame(
     from = c("A", "B", "C"),
     edge = c("-->", "---", "<->"),
-    to   = c("B", "C", "D")
+    to = c("B", "C", "D")
   )
 
   cg_df <- caugi(edges_df = edges_df)
@@ -422,7 +441,7 @@ test_that("edges_df input works for empty graph", {
   edges_df <- data.frame(
     from = character(),
     edge = character(),
-    to   = character()
+    to = character()
   )
 
   cg_df <- caugi(edges_df = edges_df)
@@ -436,7 +455,6 @@ test_that("edges_df input works for empty graph", {
 # ─────────────────────────── Errors and warnings ──────────────────────────────
 # ──────────────────────────────────────────────────────────────────────────────
 
-
 test_that("caugi errors with trailing commas", {
   expect_error(
     caugi(
@@ -563,7 +581,6 @@ test_that("edges df input is not a data.frame or data.table", {
 # ────────────────────────────── .view_to_caugi ────────────────────────────────
 # ──────────────────────────────────────────────────────────────────────────────
 
-
 test_that(".view_to_caugi works as expected", {
   cg <- caugi(
     A %-->% B,
